"""
TrueFX datasets are a particular case of CSV datasets (in fact: the default arguments
  of the CSV datasets are suitable for the TrueFX datasets).
"""


from mistra.core.intervals import Interval
from ..filesystem import CSVBackTestingProvider


class TrueFXBackTestingProvider(CSVBackTestingProvider):
    """
    A sub-class of CSV back-testing provider, suited to the format of
      TrueFX data sources.
    """

    def __init__(self, fileobj, base_timestamp, initial_buy, initial_sale, chunk_size=3600, price_precision=4):
        """
        Creates a setting to load data from a file object.
        :param fileobj: The file (name or object) to read from.
        :param base_timestamp: The base date[time] to use for the generated sources. It will be truncated to
          its month, ignoring everything else.
        :param initial_buy: The initial buy non-standardized price (as string!).
        :param initial_sale: The initial sale non-standardized price (as string!).
        :param chunk_size: The chunk size for the sources, and the reads.
        :param price_precision: The precision -pip depth, say- to standardize prices.
        """

        base_timestamp = base_timestamp.replace(day=1, hour=0, minute=0, second=0, microsecond=0)
        super().__init__(fileobj, base_timestamp, initial_buy, initial_sale, Interval.SECOND, chunk_size,
                         '%Y%m%d %H:%M:%S.%f', 1, 2, 3, price_precision)



import csv, os
from datetime import timedelta
from numpy import array
import pandas as pd
from mistra.core.intervals import Interval
from mistra.core.pricing import Candle
from mistra.core.sources import Source


def dump(source, file_pattern, interval=Interval.DAY):
    """
    Takes a source of candles and stores its contents in several chunks. The chunk size will
      be the ratio of the given interval by the source's interval. For this to work, some things
      are required to happen:

      - The given source must be of candle type.
      - The given interval must be exactly divisible by the source's interval.
      - Each chunk will be stored according to the given file pattern, which must have a format
          according to strftime/strptime datetime functions.

    Please note: THE FILES'S CONTENTS DO NOT KEEP ANY KIND OF TRACK OF TIMESTAMPS -neither base
      or per-candle timestamps- OR INTERVAL SIZES. Please make sure you state somewhere what
      interval size and base date are you dealing with when loading one of those dumps (e.g. use
      an appropriate file pattern reflecting the interval size, and use all the markers you'd need
      for the date/time). You can have for sure that the used timing is regular (i.e. all the
      candles have the same time width).

    :param source: The source to chunk.
    :param file_pattern: The destination of the files.
    :param interval: The interval the source will be chunked by. By default, one day. It could be less.
      This means: chunks will span a period of 1 day, or the given interval.
    """

    if source.dtype != Candle:
        raise ValueError("Given source must use underlying Candle types to work")
    interval_size = int(interval)
    ratio, remainder = divmod(interval_size, int(source.interval))
    if remainder:
        raise ValueError("Given interval must be exactly divisible by source's interval")
    length = len(source)
    current_chunk_date = source.timestamp
    for idx in range(0, length, ratio):
        chunk = source[idx:min(idx + ratio, length)]
        chunk.shape = chunk.shape[0:1]
        concrete_file_pattern = current_chunk_date.strftime(file_pattern)
        os.makedirs(os.path.dirname(concrete_file_pattern), 0o755, True)
        with open(current_chunk_date.strftime(file_pattern), 'w') as output:
            writer = csv.writer(output)
            for chunk_idx in range(0, chunk.shape[0]):
                candle = chunk[chunk_idx]
                writer.writerow((candle.start, candle.min, candle.max, candle.end))
        current_chunk_date += timedelta(seconds=interval_size)


def load(fileobj, interval, base_timestamp, initial=None):
    """
    Given a file name or file object, an interval, and a base timestamp (and other optional arguments)
      it loads candle data from a 4-column CSV file and makes a source frame with that.

    :param fileobj: The source file object (or name).
    :param interval: The interval to consider (beforehand known that interval is the appropriate).
    :param base_timestamp: The base timestamp to use (beforehand known the source corresponds to it).
    :param initial: The initial value (beforehand known that value was the end price in the previous market data).
    :return:
    """

    if not isinstance(initial, Candle) and initial is not None:
        initial = Candle.constant(initial)
    pd_csv = pd.read_csv(fileobj, chunksize=3600, header=None)
    source = Source(Candle, base_timestamp, interval, initial)
    rows = []
    for chunk in pd_csv:
        for idx, row in chunk.iterrows():
            rows.append(Candle(start=row[0], min=row[1], max=row[2], end=row[3]))
    data = array(rows, dtype=Candle)
    data.shape = (data.shape[0], 1)
    source.push(data)
    return source



"""
File-system based historical data (back-testing) providers exist here.

Since this is old data, file-system can safely use it.

One example: CSV data.
"""
from datetime import datetime

import pandas as pd
from mistra.core.intervals import Interval
from mistra.core.pricing import Candle
from mistra.core.providers import BackTestingProvider
from mistra.core.sources import Source


class CSVBackTestingProvider(BackTestingProvider):
    """
    Parses a CSV file and creates the two sources from it.
    """

    def __init__(self, fileobj, base_timestamp, initial_buy, initial_sale,
                 interval=Interval.SECOND, chunk_size=3600, timestamp_format='%Y%m%d %H:%M:%S.%f',
                 timestamp_column=1, buy_price_column=2, sale_price_column=3, price_precision=4):
        """
        Reads data from a CSV file.
        :param fileobj: The file (name or object) to read from.
        :param base_timestamp: The base date[time] to use for the generated sources. E.g. for truefx, it will be
          <year>-<month>-01 00:00:00.
        :param interval: The interval to srink the data into.
        :param chunk_size: The chunk size to use for the internal growing arrays.
        :param timestamp_format: The timestamp format to use when reading the timestamp column.
        :param initial_buy: Initial buy non-standardized price to use (e.g. the end price from previous work month).
        :param initial_sale: Initial sale non-standardized price to use (e.g. the end price from previous work month).
        :param timestamp_column: The column holding the timestamp.
        :param buy_price_column: The column with the buy price.
        :param sale_price_column: The column with the sale price (which will be bigger than buy price).
        :param price_precision: The precision to parse the decimals. /USD pairs must have 4 (which will
          turn into 5) while /JPY pairs must use 2 (which will turn into 3).
        """

        self._fileobj = fileobj
        self._base_timestamp = base_timestamp
        self._interval = interval
        self._chunk_size = chunk_size
        self._timestamp_format = timestamp_format
        self._timestamp_column = timestamp_column
        self._buy_price_column = buy_price_column
        self._sale_price_column = sale_price_column
        self._price_precision = price_precision
        self._initial_buy = Candle.constant(self._standardize(initial_buy))
        self._initial_sale = Candle.constant(self._standardize(initial_sale))

    def _standardize(self, price_string):
        """
        Standardizes a price, according to the current price precision.
        :return: The standardized price.
        """

        parts = price_string.split('.')
        frac_amount = int(parts[1].ljust(self._price_precision + 1, '0'))
        int_amount = int(parts[0]) * 10 ** (self._price_precision + 1)
        return int_amount + frac_amount

    def _execute(self):
        """
        Reads the CSV from a data frame.
        :return: A tuple with the (buy price, sale price).
        """

        csv = pd.read_csv(self._fileobj, chunksize=self._chunk_size, header=None,
                          dtype={self._buy_price_column: str, self._sale_price_column: str})
        buy_source = Source(Candle, self._base_timestamp, self._interval, self._initial_buy)
        sale_source = Source(Candle, self._base_timestamp, self._interval, self._initial_sale)
        for chunk in csv:
            for idx, row in chunk.iterrows():
                stamp = datetime.strptime(row[self._timestamp_column], self._timestamp_format)
                buy_price = self._standardize(row[self._buy_price_column])
                sale_price = self._standardize(row[self._sale_price_column])
                self._merge(stamp, buy_price, buy_source)
                self._merge(stamp, sale_price, sale_source)
        return buy_source, sale_source



from setuptools import setup

setup(
    name='mistra-providers',
    version='0.0.4',
    packages=['mistra.providers.historical.preprocessing',
              'mistra.providers.historical.raw.filesystem',
              'mistra.providers.historical.raw.truefx'],
    url='',
    license='MIT',
    author='luismasuelli',
    author_email='luisfmasuelli@gmail.com',
    description='MISTRA Providers contains a collection of custom providers for the MISTRA package',
    python_requires='>=3.3',
    install_requires=['mistra==0.0.5', 'pandas==0.25.3']
)
